/*
   +----------------------------------------------------------------------+
   | All rights reserved                                                  |
   |                                                                      |
   | Redistribution and use in source and binary forms, with or without   |
   | modification, are permitted provided that the following conditions   |
   | are met:                                                             |
   |                                                                      |
   | 1. Redistributions of source code must retain the above copyright    |
   |    notice, this list of conditions and the following disclaimer.     |
   | 2. Redistributions in binary form must reproduce the above copyright |
   |    notice, this list of conditions and the following disclaimer in   |
   |    the documentation and/or other materials provided with the        |
   |    distribution.                                                     |
   | 3. The names of the authors may not be used to endorse or promote    |
   |    products derived from this software without specific prior        |
   |    written permission.                                               |
   |                                                                      |
   | THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS  |
   | "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    |
   | LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS    |
   | FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE       |
   | COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,  |
   | INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, |
   | BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;     |
   | LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER     |
   | CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT   |
   | LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN    | 
   | ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE      |
   | POSSIBILITY OF SUCH DAMAGE.                                          |
   +----------------------------------------------------------------------+
   | Authors: Ryusuke Sekiyama <rsky0711@gmail.com>                       |
   +----------------------------------------------------------------------+
*/

/* $ Id: $ */ 

#include "php_cvcapture.h"

#if HAVE_CVCAPTURE

/* {{{ Resource destructors */
int le_cvcapture;
void cvcapture_dtor(zend_rsrc_list_entry *rsrc TSRMLS_DC)
{
	CvCapture * resource = (CvCapture *)(rsrc->ptr);

	do {
		cvReleaseCapture(&resource);
	} while (0);
}

/* }}} */

/* {{{ Class definitions */

/* {{{ Class CvCapture */

static zend_class_entry * CvCapture_ce_ptr = NULL;

/* {{{ Methods */


/* {{{ proto object createCameraCapture([int index])
  Start capturing frames from camera. */
PHP_METHOD(CvCapture, createCameraCapture)
{
	zend_class_entry * _this_ce;

	zval * _this_zval = NULL;
	long index = 0;



	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "O|l", &_this_zval, CvCapture_ce_ptr, &index) == FAILURE) {
		return;
	}

	_this_ce = Z_OBJCE_P(_this_zval);


	php_error(E_WARNING, "createCameraCapture: not yet implemented"); RETURN_FALSE;

	object_init(return_value)
}
/* }}} createCameraCapture */



/* {{{ proto object createFileCapture(string filename)
  Start capturing frames from video file. */
PHP_METHOD(CvCapture, createFileCapture)
{
	zend_class_entry * _this_ce;

	zval * _this_zval = NULL;
	const char * filename = NULL;
	int filename_len = 0;



	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Os", &_this_zval, CvCapture_ce_ptr, &filename, &filename_len) == FAILURE) {
		return;
	}

	_this_ce = Z_OBJCE_P(_this_zval);


	php_error(E_WARNING, "createFileCapture: not yet implemented"); RETURN_FALSE;

	object_init(return_value)
}
/* }}} createFileCapture */



/* {{{ proto bool save(string filename[, mixed &size])
  Capture a frame. */
PHP_METHOD(CvCapture, save)
{
	zend_class_entry * _this_ce;

	zval * _this_zval = NULL;
	const char * filename = NULL;
	int filename_len = 0;
	zval * size = NULL;



	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Os|z", &_this_zval, CvCapture_ce_ptr, &filename, &filename_len, &size) == FAILURE) {
		return;
	}

	_this_ce = Z_OBJCE_P(_this_zval);


	php_error(E_WARNING, "save: not yet implemented"); RETURN_FALSE;

	RETURN_FALSE;
}
/* }}} save */


static zend_function_entry CvCapture_methods[] = {
	PHP_ME(CvCapture, createCameraCapture, CvCapture__createCameraCapture_args, /**/ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
	PHP_ME(CvCapture, createFileCapture, CvCapture__createFileCapture_args, /**/ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
	PHP_ME(CvCapture, save, CvCapture__save_args, /**/ZEND_ACC_PUBLIC)
	{ NULL, NULL, NULL }
};

/* }}} Methods */

static void class_init_CvCapture(void)
{
	zend_class_entry ce;

	INIT_CLASS_ENTRY(ce, "CvCapture", CvCapture_methods);
	CvCapture_ce_ptr = zend_register_internal_class(&ce);

	/* {{{ Property registration */

	zend_declare_property_null(CvCapture_ce_ptr, 
		"capture", 7, 
		ZEND_ACC_PRIVATE TSRMLS_DC);

	/* }}} Property registration */


	/* {{{ Constant registration */

	do {
		zval *tmp, *val;
		zend_declare_class_constant_long(CvCapture_ce_ptr, "ANY", 3, CV_CAP_ANY TSRMLS_CC );
		zend_declare_class_constant_long(CvCapture_ce_ptr, "MIL", 3, CV_CAP_MIL TSRMLS_CC );
		zend_declare_class_constant_long(CvCapture_ce_ptr, "VFW", 3, CV_CAP_VFW TSRMLS_CC );
		zend_declare_class_constant_long(CvCapture_ce_ptr, "V4L", 3, CV_CAP_V4L TSRMLS_CC );
		zend_declare_class_constant_long(CvCapture_ce_ptr, "V4L2", 4, CV_CAP_V4L2 TSRMLS_CC );
		zend_declare_class_constant_long(CvCapture_ce_ptr, "FIREWARE", 8, CV_CAP_FIREWARE TSRMLS_CC );
		zend_declare_class_constant_long(CvCapture_ce_ptr, "IEEE1394", 8, CV_CAP_IEEE1394 TSRMLS_CC );
		zend_declare_class_constant_long(CvCapture_ce_ptr, "DC1394", 6, CV_CAP_DC1394 TSRMLS_CC );
		zend_declare_class_constant_long(CvCapture_ce_ptr, "CMU1394", 7, CV_CAP_FIREWARE TSRMLS_CC );
		zend_declare_class_constant_long(CvCapture_ce_ptr, "STEREO", 6, CV_CAP_STEREO TSRMLS_CC );
		zend_declare_class_constant_long(CvCapture_ce_ptr, "TYZX", 4, CV_CAP_TYZX TSRMLS_CC );
		zend_declare_class_constant_long(CvCapture_ce_ptr, "TYZX_LEFT", 9, CV_TYZX_LEFT TSRMLS_CC );
		zend_declare_class_constant_long(CvCapture_ce_ptr, "TYZX_RIGHT", 10, CV_TYZX_RIGHT TSRMLS_CC );
		zend_declare_class_constant_long(CvCapture_ce_ptr, "TYZX_COLOR", 10, CV_TYZX_COLOR TSRMLS_CC );
		zend_declare_class_constant_long(CvCapture_ce_ptr, "TYZX_Z", 6, CV_TYZX_Z TSRMLS_CC );
		zend_declare_class_constant_long(CvCapture_ce_ptr, "QT", 2, CV_CAP_QT TSRMLS_CC );
		zend_declare_class_constant_long(CvCapture_ce_ptr, "UNICAP", 6, CV_CAP_UNICAP TSRMLS_CC );
		zend_declare_class_constant_long(CvCapture_ce_ptr, "DSHOW", 5, CV_CAP_DSHOW TSRMLS_CC );
		zend_declare_class_constant_long(CvCapture_ce_ptr, "PVAPI", 5, CV_CAP_PVAPI TSRMLS_CC );
	} while(0);

	/* } Constant registration */

}

/* }}} Class CvCapture */

/* }}} Class definitions*/

/* {{{ cvcapture_functions[] */
function_entry cvcapture_functions[] = {
	PHP_FE(cv_create_camera_capture, cv_create_camera_capture_arg_info)
	PHP_FE(cv_create_file_capture, cv_create_file_capture_arg_info)
	PHP_FE(cv_save_capture     , cv_save_capture_arg_info)
	{ NULL, NULL, NULL }
};
/* }}} */


/* {{{ cvcapture_module_entry
 */
zend_module_entry cvcapture_module_entry = {
	STANDARD_MODULE_HEADER,
	"cvcapture",
	cvcapture_functions,
	PHP_MINIT(cvcapture),     /* Replace with NULL if there is nothing to do at php startup   */ 
	PHP_MSHUTDOWN(cvcapture), /* Replace with NULL if there is nothing to do at php shutdown  */
	PHP_RINIT(cvcapture),     /* Replace with NULL if there is nothing to do at request start */
	PHP_RSHUTDOWN(cvcapture), /* Replace with NULL if there is nothing to do at request end   */
	PHP_MINFO(cvcapture),
	PHP_CVCAPTURE_VERSION, 
	STANDARD_MODULE_PROPERTIES
};
/* }}} */

#ifdef COMPILE_DL_CVCAPTURE
ZEND_GET_MODULE(cvcapture)
#endif


/* {{{ PHP_MINIT_FUNCTION */
PHP_MINIT_FUNCTION(cvcapture)
{
	REGISTER_LONG_CONSTANT("CV_CAP_ANY", CV_CAP_ANY, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_MIL", CV_CAP_MIL, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_VFW", CV_CAP_VFW, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_V4L", CV_CAP_V4L, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_V4L2", CV_CAP_V4L2, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_FIREWARE", CV_CAP_FIREWARE, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_IEEE1394", CV_CAP_IEEE1394, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_DC1394", CV_CAP_DC1394, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_CMU1394", CV_CAP_FIREWARE, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_STEREO", CV_CAP_STEREO, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_TYZX", CV_CAP_TYZX, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_TYZX_LEFT", CV_TYZX_LEFT, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_TYZX_RIGHT", CV_TYZX_RIGHT, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_TYZX_COLOR", CV_TYZX_COLOR, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_TYZX_Z", CV_TYZX_Z, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_QT", CV_CAP_QT, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_UNICAP", CV_CAP_UNICAP, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_DSHOW", CV_CAP_DSHOW, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_PVAPI", CV_CAP_PVAPI, CONST_PERSISTENT | CONST_CS);
	le_cvcapture = zend_register_list_destructors_ex(cvcapture_dtor, 
						   NULL, "cvcapture", module_number);
	class_init_CvCapture();

	/* add your stuff here */

	return SUCCESS;
}
/* }}} */


/* {{{ PHP_MSHUTDOWN_FUNCTION */
PHP_MSHUTDOWN_FUNCTION(cvcapture)
{

	/* add your stuff here */

	return SUCCESS;
}
/* }}} */


/* {{{ PHP_RINIT_FUNCTION */
PHP_RINIT_FUNCTION(cvcapture)
{
	/* add your stuff here */

	return SUCCESS;
}
/* }}} */


/* {{{ PHP_RSHUTDOWN_FUNCTION */
PHP_RSHUTDOWN_FUNCTION(cvcapture)
{
	/* add your stuff here */

	return SUCCESS;
}
/* }}} */


/* {{{ PHP_MINFO_FUNCTION */
PHP_MINFO_FUNCTION(cvcapture)
{
	php_printf("Capture frames from webcams.\n");
	php_info_print_table_start();
	php_info_print_table_row(2, "Version",PHP_CVCAPTURE_VERSION " (beta)");
	php_info_print_table_row(2, "Released", "2011-01-26");
	php_info_print_table_row(2, "CVS Revision", "$Id: $");
	php_info_print_table_row(2, "Authors", "Ryusuke Sekiyama 'rsky0711@gmail.com' (lead)\n");
	php_info_print_table_end();
	/* add your stuff here */

}
/* }}} */


/* {{{ proto resource cvcapture cv_create_camera_capture([int index])
  Start capturing frames from camera. */
PHP_FUNCTION(cv_create_camera_capture)
{
	CvCapture * return_res;
	long return_res_id = -1;

	long index = 0;



	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|l", &index) == FAILURE) {
		return;
	}

	php_error(E_WARNING, "cv_create_camera_capture: not yet implemented"); RETURN_FALSE;

	ZEND_REGISTER_RESOURCE(return_value, return_res, le_cvcapture);
}
/* }}} cv_create_camera_capture */


/* {{{ proto resource cvcapture cv_create_file_capture(string filename)
  Start capturing frames from video file. */
PHP_FUNCTION(cv_create_file_capture)
{
	CvCapture * return_res;
	long return_res_id = -1;

	const char * filename = NULL;
	int filename_len = 0;



	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &filename, &filename_len) == FAILURE) {
		return;
	}

	php_error(E_WARNING, "cv_create_file_capture: not yet implemented"); RETURN_FALSE;

	ZEND_REGISTER_RESOURCE(return_value, return_res, le_cvcapture);
}
/* }}} cv_create_file_capture */


/* {{{ proto bool cv_save_capture(resource cvcapture capture, string filename[, mixed &size])
  Capture a frame. */
PHP_FUNCTION(cv_save_capture)
{
	zval * capture = NULL;
	int capture_id = -1;
	CvCapture * res_capture;

	const char * filename = NULL;
	int filename_len = 0;
	zval * size = NULL;



	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs|z", &capture, &filename, &filename_len, &size) == FAILURE) {
		return;
	}
	ZEND_FETCH_RESOURCE(res_capture, CvCapture *, &capture, capture_id, "cvcapture", le_cvcapture);



	php_error(E_WARNING, "cv_save_capture: not yet implemented"); RETURN_FALSE;

	RETURN_FALSE;
}
/* }}} cv_save_capture */

#endif /* HAVE_CVCAPTURE */


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
